<?php
/*
 * Plugin Name: Maintenance Mode Lite
 * Plugin URI: https://codiant.com/services/wordpress-development/
 * Description: Enables a customizable maintenance mode for your website. Allows scheduling and personalized messages to visitors.
 * Version: 1.0.1
 * Author: Codiant Team
 * Author URI: https://codiant.com/
 * Requires at least: 6.4
 * Tested up to: 6.7
 * Requires PHP: 7.4
 * Text Domain: maintenance-mode-lite
 * Domain Path: /languages/
 * License: GPL2+
 * 
 * @package maintenance-mode-lite
 */

if (!defined('ABSPATH')) {
    exit;
}

// Define constants with a unique prefix
define('DTMML_PLUGIN_DIR', untrailingslashit(plugin_dir_path(__FILE__)));
define('DTMML_PLUGIN_URL', untrailingslashit(plugins_url(basename(plugin_dir_path(__FILE__)), basename(__FILE__))));
define('DTMML_PLUGIN_BASENAME', plugin_basename(__FILE__));

// 1. Add the settings page to the admin menu
if (!function_exists('dtmml_add_settings_menu')) {
    function dtmml_add_settings_menu() {
        add_options_page(
            __('Maintenance Mode Settings', 'maintenance-mode-lite'),
            __('Maintenance Mode', 'maintenance-mode-lite'),
            'manage_options',
            'dtmml-maintenance-mode',
            'dtmml_render_settings_page'
        );
    }
}
add_action('admin_menu', 'dtmml_add_settings_menu');

// 2. Register settings
if (!function_exists('dtmml_register_settings')) {
    function dtmml_register_settings() {
        register_setting(
            'dtmml_settings_group',
            'dtmml_maintenance_message',
            array(
                'type'              => 'string',
                'sanitize_callback' => 'sanitize_text_field', // Sanitizes the message input
            )
        );

        register_setting(
            'dtmml_settings_group',
            'dtmml_maintenance_start_time',
            array(
                'type'              => 'string',
                'sanitize_callback' => 'sanitize_text_field', // Sanitizes the start time input
            )
        );

        register_setting(
            'dtmml_settings_group',
            'dtmml_maintenance_end_time',
            array(
                'type'              => 'string',
                'sanitize_callback' => 'sanitize_text_field', // Sanitizes the end time input
            )
        );
    }
}
add_action('admin_init', 'dtmml_register_settings');

// 3. Render the settings page
if (!function_exists('dtmml_render_settings_page')) { 
    function dtmml_render_settings_page() {
        ?>
        <div class="wrap">
            <h1><?php echo esc_html(__('Maintenance Mode Settings', 'maintenance-mode-lite')); ?></h1>
            <form method="post" action="options.php">
                <?php
                settings_fields('dtmml_settings_group');
                do_settings_sections('dtmml_settings_group');
                ?>
                <table class="form-table">
                    <tr valign="top">
                        <th scope="row"><?php echo esc_html(__('Maintenance Message', 'maintenance-mode-lite')); ?></th>
                        <td>
                            <textarea name="dtmml_maintenance_message" rows="5" cols="50"><?php echo esc_textarea(get_option('dtmml_maintenance_message', __('We are currently under maintenance. Please check back soon!', 'maintenance-mode-lite'))); ?></textarea>
                        </td>
                    </tr>
                    <tr valign="top">
                        <th scope="row"><?php echo esc_html(__('Start Time', 'maintenance-mode-lite')); ?></th>
                        <td>
                            <input type="datetime-local" name="dtmml_maintenance_start_time" value="<?php echo esc_attr(get_option('dtmml_maintenance_start_time')); ?>">
                        </td>
                    </tr>
                    <tr valign="top">
                        <th scope="row"><?php echo esc_html(__('End Time', 'maintenance-mode-lite')); ?></th>
                        <td>
                            <input type="datetime-local" name="dtmml_maintenance_end_time" value="<?php echo esc_attr(get_option('dtmml_maintenance_end_time')); ?>">
                        </td>
                    </tr>
                </table>
                <?php submit_button(__('Save Settings', 'maintenance-mode-lite')); ?>
            </form>
        </div>
        <?php
    }
}

// 4. Display maintenance mode based on schedule
if (!function_exists('dtmml_maintenance_mode')) { 
    function dtmml_maintenance_mode() {
        if (current_user_can('manage_options')) {
            return;
        }
        
        $start_time = strtotime(get_option('dtmml_maintenance_start_time'));
        $end_time = strtotime(get_option('dtmml_maintenance_end_time'));
        $current_time = current_time('timestamp');

        if ($start_time && $end_time) {
            if ($current_time >= $start_time && $current_time <= $end_time) {
                $maintenance_message = get_option('dtmml_maintenance_message', __('We are currently under maintenance. Please check back soon!', 'maintenance-mode-lite'));
                wp_die(
                    '<div style="text-align: center; margin-top: 50px;">
                        <h1 style="font-size: 48px; color: #333;">' . esc_html(__('Maintenance Mode', 'maintenance-mode-lite')) . '</h1>
                        <p style="font-size: 24px; color: #666;">' . esc_html($maintenance_message) . '</p>
                    </div>',
                    esc_html(__('Maintenance Mode', 'maintenance-mode-lite')),
                    array('response' => '503')
                );
            }
        }
    }
}
add_action('get_header', 'dtmml_maintenance_mode');